require 'rubygems'
require 'selenium-webdriver'

require_relative  "../module/DriverUtility"
require_relative "date_filter"

class Diet
  include DriverUtility

  TITLE = ".main-title[aria-hidden]"
  EVENT_TITLE = "eventTitle"

  def initialize(driver)
    @driver = driver
    @date_filter = Date_Filter.new(@driver) 
  end

  def getTitle()
    return getTextForElement(:css, TITLE)
  end

  def clickAddMealBtn()
    click(:css, "button[ng-click='addItem()']")
  end

  def clickSaveNewBtn()
    click(:css, "button[ng-click='saveNewItem()']")
  end

  def clickCancelBtn()
    click(:css, "button[ng-click='cancel()']")
  end

  def clickSaveExistingMealBtn()
    click(:css, "button[ng-click='saveItem()']")
  end

  def clickDeleteBtn()
    click(:css, "button[ng-click='deleteItem()']")
  end

  def clickNthMeal(number)
    click(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button")
  end

  def getPrimaryHeader()
    return getTextForElement(:css, "div.primary-header h2")
  end

  def getSecondaryHeader()
    return getTextForElement(:css, "div.secondary-header h2")
  end

  def selectType(type)
    select = getElement(:css, "select[name='mealType']")
    options = select.find_elements(:tag_name, "option")
    options.each do |option|
      if(option.attribute('value') == type)
        option.click
        break
      end
    end
  end

  def setCalories(number)
    @driver.find_element(:css, "input[name='caloriesInput']").clear
    @driver.find_element(:css, "input[name='caloriesInput']").send_keys(number)
  end

  def setFat(number)
    @driver.find_element(:css, "input[name='fatInput']").clear
    @driver.find_element(:css, "input[name='fatInput']").send_keys(number)
  end

  def setCarb(number)
    @driver.find_element(:css, "input[name='carbInput']").clear
    @driver.find_element(:css, "input[name='carbInput']").send_keys(number)
  end

  def setProtein(number)
    @driver.find_element(:css, "input[name='proteinInput']").clear
    @driver.find_element(:css, "input[name='proteinInput']").send_keys(number)
  end

  def setNarrative(text)
    @driver.find_element(:css, "textarea[name='Note']").clear
    @driver.find_element(:css, "textarea[name='Note']").send_keys(text)
  end

  def setDateRange(fromDate, toDate)
    @date_filter.setDateRangeAndFilter(fromDate, toDate)
    sleep 5
  end

  def getNoResultsFoundMessage()
    return getTextForElement(:css, '.no-results-message')
  end
  
  def verifyDefautDateRangeIs30DaysBack()
    verifiedYes = false
    todayDate = getDateNthDaysAgo(0, "%m/%d/%Y")
    thirtyDaysAgo = getDateNthDaysAgo(30, "%m/%d/%Y")
    fromDate = getTextFromInput(:css, "input[name='startDate']")

    toDate = getTextFromInput(:css, "input[name='endDate']")
    if toDate == todayDate and fromDate == thirtyDaysAgo then
      verifiedYes = true
    end
  end

  def isDateWithinFilteredDateRange?()
    fromDate = getTextFromInput(:css, "input[name='startDate']")
    toDate = getTextFromInput(:css, "input[name='endDate']")
    #puts "[isDateWithinFilteredDateRange?()] Filter Date Range " + fromDate + " - " + toDate

    mealCount = getTotalMealCount()
    isValid = true

    fromDateObj = DateTime.strptime(fromDate, "%m/%d/%Y")
    toDateObj = DateTime.strptime(toDate, "%m/%d/%Y")

    for i in 1..mealCount
      mealDateTime = getNthDateTimeInMealList(i)
      mealDateArry = mealDateTime.split(" ")
      mealDate = mealDateArry[0]
      mealDateObj = DateTime.strptime(mealDate, "%m/%d/%Y")

      #puts "mealDate " + mealDate
      if mealDateObj <= fromDateObj and mealDateObj >= toDateObj then
        isValid = false
        break
      end
    end

    return isValid

  end

  def getNthDateTimeInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(1) div:nth-of-type(2)")
  end

  def getNthCaloriesInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(2) div:nth-of-type(1)")
  end

  def getNthTypeInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(1) div:nth-of-type(1) strong")
  end

  def getNthFatInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(2) div:nth-of-type(2)")
  end

  def getNthCarbInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(3) div:nth-of-type(1)")
  end

  def getNthProteinInMealList(number)
    return getTextForElement(:css, "ul.secondary-list li:nth-of-type(" + number.to_s + ") button div:nth-of-type(3) div:nth-of-type(2)")
  end

  def getNthLabelCalories()
    return getTextForElement(:css, "#caloriesInput div label span span:nth-of-type(1)")
  end

  def getNthLabelFat()
    return getTextForElement(:css, "#fatInput div label span span:nth-of-type(1)")
  end

  def getNthLabelCarbs()
    return getTextForElement(:css, "#carbInput div label span span:nth-of-type(1)")
  end

  def getNthLabelProtein()
    return getTextForElement(:css, "#proteinInput div label span span:nth-of-type(1)")
  end

  def getMessageInEventDetail()
    return getTextForElement(:css, "div[ng-if='!details'] span")
  end

  def getTotalMealCount()
    begin
      elements = getElements(:css, "ul.secondary-list li strong")
      return elements.length
    rescue Exception=>e
      puts "Exception " + e.to_s
      return 0
    end
  end

  def findIndexWithAGivenMealType(mealType)
    mealLength = getTotalMealCount()
    #puts "Meal Length : " + mealLength.to_s
    index = -1

    for i in 1..mealLength
      #puts "listtype = " + getNthTypeInMealList(i)
      listtype = getNthTypeInMealList(i)
      if listtype == mealType then
        index = i
        break
      end
    end
    #puts "rowIndex found : " + index.to_s
    return index
  end

  def isThisMealDisplayed?(mealType)
    isDisplaying = false
    rowIndex = findIndexWithAGivenMealType(mealType)
    if(rowIndex > 0)
      isDisplaying = true
    end
    return isDisplaying
  end

  def addAMeal(dateStr, timeStr, type, calories, fat, carb, protein, text)
    clickAddMealBtn()
    !45.times{ break if (getPrimaryHeader() == "Meal Detail"); sleep 1 }
    getPrimaryHeader().should == "Meal Detail"

    setDateAndTime(dateStr, timeStr, "input[name='date']", "input[name='time']")
    sleep 0.5
    selectType(type)
    sleep 0.2
    setCalories(calories)
    sleep 0.2
    setFat(fat)
    sleep 0.2
    setCarb(carb)
    sleep 0.2
    setProtein(protein)
    sleep 0.2
    setNarrative(text)
    sleep 0.2
    clickSaveNewBtn()
  end

  def verifyTotalCharacterInNarrative()
    boundaryTest = false
    part1 = "VerifyTheTotalAllowedCharactersInNarrativeField001"
    part2 = "VerifyTheTotalAllowedCharactersInNarrativeField002"
    part3 = "VerifyTheTotalAllowedCharactersInNarrativeField003"
    part4 = "VerifyTheTotalAllowedCharactersInNarrativeField004"
    part5 = "VerifyTheTotalAllowedCharactersInNarrativeField005"
    typeInText = part1 + part2+ part3 + part4 + part5

    setNarrative(typeInText)

    counterText = getTextForElement(:css, "span[ng-if='characterCounter']")
    if counterText == "0 characters left"
      boundaryTest = true
    else
      boundaryTest = false
    end

    return boundaryTest
  end

  def EditAMeal(number)
    setCalories(number)
    clickSaveExistingMealBtn()
  end

#"%Y-%m-%d"
  def getDateNthDaysAgo(numberOfDaysAgo, formatStr)
    dateNthDaysAgo = numberOfDaysAgo.day.ago.strftime(format=formatStr)
    return dateNthDaysAgo

  end

end