require 'rubygems'
require 'selenium-webdriver'

require_relative  "../module/DriverUtility"
require_relative  "../helpers/table"
require_relative "../helpers/graph"

class PersonalTrackers
  include DriverUtility
  include Table
  include Graph

  PDF_BTN = "button[title='PDF']"
  GRAPH_BTN = "button[title='Graph']"
  TABLE_BTN = "button[title='Table']"
  ADD_ENTRY_BTN = "button[ng-click='getDetails()']"
  NOTIFICATION_ICON = "i.icon-notification"

  def initialize(driver, metaData)
    @driver = driver
    if(metaData != nil)
      @columnList = metaData["orderedColumnsList"];
      @columnMap = Hash.new();

      columnListLength = @columnList.length

      for i in 0...columnListLength
        @columnMap[@columnList[i]] = i + 1
      end
    end
  end

  def clickAddEntryButton()
    click(:css, ADD_ENTRY_BTN)
  end

  def getRowValuesForSection(row)
    return getDataForRow(@columnList, row)
  end

  def getTextForGridCell(columnKey, row)
    return getTextForRowColumn(row, @columnMap[columnKey]);
  end

  def isPDFButtonDisplayed?()
    return isElementVisible(:css, PDF_BTN)
  end

  def isGraphButtonDisplayed?()
    return isElementVisible(:css, GRAPH_BTN)
  end

  def isTableButtonDisplayed?()
    return isElementVisible(:css, TABLE_BTN)
  end

  def clickPDFButton()
    click(:css, PDF_BTN)
  end

  def clickGraphButton()
    click(:css, GRAPH_BTN)
  end

  def clickTableButton()
    click(:css, TABLE_BTN)
  end

  def hasEmbeddedPDF?()
    urlLink = @driver.current_url
    return urlLink.include?("pdf")
  end


  def clickNotificationButton()
   click(:css, "[ng-click='button.onClick()'][title*='Notification']")
  end

  def getNotificationButtonLabel()
   return getTextForElement(:css, "[ng-click='button.onClick()'][title*='Notification']")
  end

  def getNotificationButtonAltText()
    return getElement(:css, "[ng-click='button.onClick()'][title*='Notification']").attribute("aria-label")
  end


end